﻿/// Copyright (c) 2012 John Tear
///
/// Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
/// and associated documentation files (the "Software"), to deal in the Software without restriction, 
/// including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
/// and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, 
/// subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in all copies or substantial 
/// portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
/// LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN 
/// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
/// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
/// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///

using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

using com.feelgoodsoftware.sharpsyslog.utilities;

namespace com.feelgoodsoftware.sharpsyslog
{
    /// <summary>
    /// The facility value in the PRI field.
    /// </summary>
    public enum SysLogFacility
    {
        KernelMessages = 0,
        UserLevelMessages = 1,
        MailSystem = 2,
        SystemDaemons = 3,
        SecAuthMessages4 = 4,
        Internal = 5,
        LinkPrinterSubSystem = 6,
        NetworkNewsSubSystem = 7,
        UUCPSubSystem = 8,
        ClockDaemon9 = 9,
        SecAuthMessages10 = 10,
        FTPDaemon = 11,
        NTPSubSystem = 12,
        LogAudit = 13,
        LogAlert = 14,
        ClockDaemon15 = 15,
        Local0 = 16,
        Local1 = 17,
        Local2 = 18,
        Local3= 19,
        Local4 = 20,
        Local5 = 21,
        Local6 = 22,
        Local7 = 23
    }

    /// <summary>
    /// Denotes the severity of a SysLog message, found in the PRI field.
    /// </summary>
    public enum SysLogSeverity
    {
        Emergency = 0,
        Alert = 1,
        Critical = 2,
        Error = 3,
        Warning = 4,
        Notice = 5,
        Informational = 6,
        Debug = 7
    }


    /// <summary>
    /// Class to represent a syslog packet
    /// </summary>
    public class SyslogPacket
    {
        /// <summary>
        /// Facility.
        /// </summary>
        public SysLogFacility Facility { get; set; }

        /// <summary>
        /// Severity.
        /// </summary>
        public SysLogSeverity Severity { get; set; }

        /// <summary>
        /// The message in hex format.
        /// </summary>
        public string DataAsHex { get; set; }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="Data">The data received.</param>
        public SyslogPacket(byte[] Data)
        {
            // todo: formulate a packet here...
            string DataAsASCIIString = Encoding.ASCII.GetString(Data);

            // dig out the pri code
            Regex regEx = new Regex("<*[0-9]*?>");
            Match match = regEx.Match(DataAsASCIIString);
            string priCode = match.Value;
            priCode = priCode.Substring(1, priCode.Length - 2);

            // calculate the pri code elements
            Facility = (SysLogFacility)CalculateFacilityValue(Int32.Parse(priCode));
            Severity = (SysLogSeverity)CalculateSeverityValue(Int32.Parse(priCode));

            // now the data
            byte[] subData = new byte[Data.Length - priCode.Length];

            for(int i = priCode.Length; i < Data.Length; i++)
            {
                subData[i - priCode.Length] = Data[i];
            }

            DataAsHex = ByteArray.ToHex(subData);
        }

        // calculates the facility value based upon the prival field in the packet
        private int CalculateFacilityValue(int Prival)
        {
            return (Prival - (Prival % 8)) / 8;
        }

        // calculates the severity value based upon the prival field in the packet
        private int CalculateSeverityValue(int Prival)
        {
            return Prival % 8;
        }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendFormat("Packet ID\n");
            sb.AppendFormat("+--Severity: {0}\n", Severity);
            sb.AppendFormat("+--Facility: {0}\n", Facility);
            sb.AppendFormat("+--Data: 0x{0}\n", DataAsHex);
            sb.AppendFormat("+--Data (ASCII): {0}\n", Encoding.ASCII.GetString(ByteArray.FromHex(DataAsHex)));

            return sb.ToString();
        }
    }
}
